@@ -3,6 +3,7 @@
 from typing import Iterator, Literal, Mapping, Sequence
 
 from .dfs import depth_first_search
+from .johnson import johnson
 from .modules import Module
 from .tarjan import strongly_connected_components
 
@@ -15,4 +16,6 @@ def detect_cycles(
         return depth_first_search(graph)
     if strategy == "tarjan":
         return (scc for scc in strongly_connected_components(graph) if len(scc) > 1)
+    if strategy == "johnson":
+        return johnson(graph)
     raise NotImplementedError()
