@@ -2,8 +2,19 @@
 
 from __future__ import annotations
 
+import abc
 from pathlib import Path
-from typing import Final, NamedTuple, Sequence
+from typing import Final, Sequence
+
+_INIT_NAME = "__init__"
+
+
+def _make_init_module_path(path: Path) -> Path:
+    return path / f"{_INIT_NAME}.py"
+
+
+def _make_init_module_name(name: ModuleName) -> ModuleName:
+    return name.joinname(_INIT_NAME)
 
 
 class ModuleName:
@@ -37,6 +48,12 @@ class ModuleName:
             return NotImplemented
         return self._parts > other._parts
 
+    def __le__(self, other: object) -> bool:
+        return self < other or self == other
+
+    def __ge__(self, other: object) -> bool:
+        return self > other or self == other
+
     @property
     def parts(self) -> Sequence[str]:
         return self._parts
@@ -53,22 +70,66 @@ class ModuleName:
         return ModuleName(*self._parts, *names)
 
 
-class RegularPackage(NamedTuple):
-    path: Path
-    name: ModuleName
+class Module(abc.ABC):
+    def __init__(self, *, path: Path, name: ModuleName) -> None:
+        self._validate(path, name)
+        self.path = path
+        self.name = name
+
+    @abc.abstractmethod
+    def _validate(self, path: Path, name: ModuleName) -> None:
+        raise NotImplementedError()
+
+    def __hash__(self) -> int:
+        return hash(self.name)
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, Module):
+            return NotImplemented
+        return self.name == other.name
+
+    def __lt__(self, other: object) -> bool:
+        if not isinstance(other, Module):
+            return NotImplemented
+        return self.name < other.name
+
+    def __gt__(self, other: object) -> bool:
+        if not isinstance(other, Module):
+            return NotImplemented
+        return self.name > other.name
+
+    def __le__(self, other: object) -> bool:
+        return self < other or self == other
+
+    def __ge__(self, other: object) -> bool:
+        return self > other or self == other
 
 
-class NamespacePackage(NamedTuple):
-    path: Path
-    name: ModuleName
+class RegularPackage(Module):
+    def _validate(self, path: Path, name: ModuleName) -> None:
+        if path.with_suffix("").name != _INIT_NAME:
+            raise ValueError(path)
+
+        if not name.parts or name.parts[-1] != _INIT_NAME:
+            raise ValueError(name)
 
 
-class PyModule(NamedTuple):
-    path: Path
-    name: ModuleName
+class NamespacePackage(Module):
+    def _validate(self, path: Path, name: ModuleName) -> None:
+        if path.with_suffix("").name == _INIT_NAME:
+            raise ValueError(path)
+
+        if name.parts and name.parts[-1] == _INIT_NAME:
+            raise ValueError(name)
 
 
-Module = RegularPackage | NamespacePackage | PyModule
+class PyModule(Module):
+    def _validate(self, path: Path, name: ModuleName) -> None:
+        if path.with_suffix("").name == _INIT_NAME:
+            raise ValueError(path)
+
+        if name.parts and name.parts[-1] == _INIT_NAME:
+            raise ValueError(name)
 
 
 class ModuleFactory:
@@ -80,12 +141,12 @@ class ModuleFactory:
     def _find_package_names(project_path: Path, packages: list[Path]) -> dict[str, Path]:
         pkgs_names: dict[str, Path] = {}
         for pkg in packages:
-            if (project_path / pkg / "__init__.py").exists():
+            if _make_init_module_path(project_path / pkg).exists():
                 pkgs_names.setdefault(pkg.name, pkg)
                 continue
 
             for parent in pkg.parents[::-1]:
-                if (project_path / parent / "__init__.py").exists():
+                if _make_init_module_path(project_path / parent).exists():
                     pkgs_names.setdefault(parent.name, pkg)
                     break
 
@@ -110,10 +171,10 @@ class ModuleFactory:
         module_path = self._project_path.joinpath(_get_sanitized_rel_module_path(module_name))
 
         if module_path.is_dir():
-            if (init_module_path := module_path / "__init__.py").exists():
+            if (init_module_path := _make_init_module_path(module_path)).exists():
                 return RegularPackage(
                     path=init_module_path,
-                    name=module_name.joinname("__init__"),
+                    name=_make_init_module_name(module_name),
                 )
 
             return NamespacePackage(
@@ -147,7 +208,7 @@ class ModuleFactory:
             )
 
         if module_path.is_file() and module_path.suffix == ".py":
-            if module_path.stem == "__init__":
+            if module_path.stem == _INIT_NAME:
                 return RegularPackage(
                     path=module_path,
                     name=module_name,
